# Feature requirements
#  request
#  response
#  server

namespace eval asp {

    variable input

    ##
    ## asp::errormessage message details
    ##
    ##  Print an HTML error message to stdout.
    ##
    
    proc errormessage { title message detail } {
	
	set result ""
	
	append result "<h1>$title</h1>"
	append result "<hr>"
	append result "<h3>$message</h3>"
	append result "<xmp>"
	append result "$detail\n"
	append result "</xmp>"
	
	return $result
    }

    ##
    ## asp::executefile
    ##
    ##  Read a piece of ASP enhanced HTML code from the specified file and 
    ##  execute it.
    ##
    
    proc executefile { file } {

	if [catch { server::cache $file } result ] {
	    return [asp::errormessage "Tcl Error" \
		    "Can't open asp file for reading" \
		    "file: $file\nerror: $result"]
	} else {
	    return [asp::executecode $result]
	}

	# Slower version that actually opens the data off disk instead
	# of using the webserver cache.
	#
	#if [catch { open $file r } result] {
	#    
	#    return [asp::errormessage "Tcl Error" \
	#	    "Can't open asp file for reading" \
	#	    "file: $file\nerror: $result"]
	#    
	#} else {
	#    
	#    set fd $result
	#    
	#    set content [read $fd]
	#    close $fd
	#    
	#    return [asp::executecode $content]
	#    
	#}
	
    }

    ##
    ## asp::executecode
    ##
    ##  Execute a piece of ASP enhanced HTML code. 
    ##

    ## TODO: this isn't very efficient, especially on large files...
    
    proc executecode { html } {

	global asp_result

	set code "global asp_result\n"
	append code "set asp_result {}\n"
	
	regsub -all <% $html "\x81\x82" html
	regsub -all %> $html \x81 html
	
	set parts [split $html \x81]
	foreach part $parts {
	    
	    if { [string index $part 0] == "\x82" } {
		
		# Remove the leading \x82 that marks this as tcl code 
                # and white space
		set part [string trimleft $part \x82]
		set part [string trimleft $part]
		
		# Is this a <%=...%> style piece of code?
		if { [string index $part 0] == "=" } {
		    
		    # Remove the leading = and white space
		    set part [string trimleft $part \=]
		    set part [string trim $part]
		    
		    append code "append asp_result $part\n"
		    
		} else {
		    append code "$part\n"
		}
		
	    } else {
		append code "append asp_result {$part}\n\n"
	    }
	    
	}
	
	append code "set asp_result\n"

	## ---
	if { [catch { eval $code } result] && \
		$result != "asp::error::Stop parse" } {
	    global errorInfo
	    
	    return [asp::errormessage "Asp Error" \
		    "Error while executing the page" "$result\n\nStack trace was $errorInfo\n\nCode was
	    \n$code\n"]
	    
	} else {
	    return $asp_result
	}
    }

    ##
    ## asp::write
    ##
    ##  Write output to the client. In case of .asp scripts this simply
    ##  appends the data to the result which is then sent in one big
    ##  chunk.
    ##
    
    proc write { a } {
	global asp_result
	append asp_result "$a"
	return;
    }

    ##
    ## asp::reset
    ##
    ## Deletes all the output generated so far
    ##

    proc reset {} {
	global asp_result
	set asp_result ""
	return
    }

    ##
    ## asp::stop
    ##
    ## Stops the processing of the ASP script at this point and returns the
    ## information generated so far
    ##

    proc stop {} {
	# Throw our exception to unwind that stack
	error "asp::error::Stop parse"
    }


    ##
    ## Returns to the client an authenticate response with the given realm,
    ## does not return
    ##

    proc authenticate { realm } {
	response::setheader "WWW-Authenticate" "basic realm=\"$realm\""
	response::setheader "Status" "401 Authenticate"

	# Eat any contents we've already produced
	asp::reset
	asp::write "<html>\
		<head><title>Authentication required</title><head>\n\
		<body bgcolor=#ffffff>\n\
		<font size=+5>Authentication required:</font><br>\n\
		Security clearance needed for $realm.\
		</body></html>"
	asp::stop
    }


    ##
    ## Redirects a client to another page, does not return
    ##

    proc redirect { location } {

	## Returns a full location of the admin server, e.g 
	## http://www.foo.com:9090 or https://www.bar.com
	
	proc location {} {
	    set code ""
	    catch { set code $request::env(HTTPS) }
	    if { $code == "on" } {
		set mylocation "https://"
	    } else {
		set mylocation "http://"
	    }
	    
	    # Lets check for a Host: header and use in preference
	    set hostname ""
	    catch { set hostname $request::env(HTTP_HOST) }
	    if { $hostname == "" } {
		append mylocation $request::env(SERVER_NAME)
	    } else {
		# Strip off port if its there
		regexp "\[^:]*" $hostname hostname
		append mylocation $hostname
	    }
	    if {    ( $request::env(SERVER_PORT) != "80" && \
		    $code != "on" ) || \
		    ( $request::env(SERVER_PORT) != "443" && \
		    $code == "on" ) } {
		append mylocation ":$request::env(SERVER_PORT)"
	    }

	    return $mylocation
	}

	response::setheader "Status" "302 Moved Temporarily"
	if {    [string match "http://*"  $location ] || \
		[string match "https://*" $location ] } {
	    set mylocation $location
	} else {
	    if { [string match "/*" $location ] } {
		set location [string range $location 1 end ]
	    }
	    set mylocation "[location]/$location"
	}
	response::setheader "Location" $mylocation
	asp::reset
	asp::write "<html>\
		<head><title>Moved</title></head>\n\
		<body bgcolor=#ffffff>\n\
		That page has moved to <a href=\"$mylocation\">$mylocation</a>\
		</body></html>"
	asp::stop
    }

    ##
    ## Sends the user back to the previous page, or "/" if there is
    ## no refererer
    ##
    
    proc back {} {
	set back "/"
	catch { set back $request::env(HTTP_REFERER) }
	redirect $back
    }

    ##
    ## Includes the source of another file (doesn't parse it for ASP)
    ##
    ## File is relative to the directory of the current file being parsed.

    proc include { file } {
	set filename [ file join \
		[ file dirname $request::env(SCRIPT_FILENAME)] $file ]

	if [catch { server::cache $filename } result ] {
	    asp::write "<br><i><b>Unable to include file $filename : 
	    $result</b></i><br>"
	} else {
	    asp::write $result
	}
    }

    ###################################################################
    ##
    ## Quoting support
    ##

    proc unquote_input { buf } {
	# rewrite "+" back to space
	regsub -all {\+} $buf { } buf
	# protect \ from quoting another \ and throwing off 
	# other things (first!)
	# protect $ from doing variable expansion
	# protect [ from doing evaluation
	# protect " from terminating string
	regsub -all {([\\["$])} $buf {\\\1} buf
	
	# replace line delimiters with newlines
	regsub -all -nocase "%0d%0a" $buf "\n" buf
	# Mosaic sends just %0A.  This is handled in the next command.
	
	# prepare to process all %-escapes 
	regsub -all -nocase {%([a-f0-9][a-f0-9])} $buf {[format %c 0x\1]} buf
	# process %-escapes and undo all protection
	eval return \"$buf\"
    }

    # return string but with html-special characters escaped,
    # necessary if you want to send unknown text to an html-formatted page.
    proc quote_html {s} {
	regsub -all {&}	$s {\&amp;}	s	;# must be first!
	regsub -all {"}	$s {\&quot;}	s
	regsub -all {<}	$s {\&lt;}	s
	regsub -all {>}	$s {\&gt;}	s
	return $s
    }

    # return string quoted appropriately to appear in a url
    proc quote_url {in} {
	regsub -all {%}  $in "%25" in
	regsub -all { }  $in "%20" in
	regsub -all {\?} $in "%3f" in
	regsub -all {\+} $in "%2b" in
	return $in
    }

    ###################################################################
    ##
    ## Argument handling
    ##

    proc decode_input {} {
	set input [request::data]
	set pairs [split $input &]
	foreach pair $pairs {
	    if {0 == [regexp (.*)=(.*) $pair dummy varname val]} {
		# if no match, unquote and leave it at that
		# this is typical of <isindex>-type queries
		set varname anonymous
		set val $pair
	    }
	    set varname [unquote_input $varname]
	    set val     [unquote_input $val]
	    set_uservar $varname $val
	}
	# TODO: could decode cookies here
    }

    ## Internal routine to add form variables to the asp::input(xxx) array
    proc set_uservar {varname val} {
	variable input

	set exists [info exists input($varname)]
		
	# handle lists of values correctly
	if [ regexp List$ $varname ] {
	    lappend input($varname) $val
	} else {
	    if $exists {
		error "Multiple definitions of $varname encountered in input.
		If you're trying to do this intentionally (such as with
		select), the variable must have a \"List\" suffix."
	    } else {
		set input($varname) $val
	    }
	}
    }

    ########################################################################
    ## 
    ## Debugging
    ##

    proc dump_input {} {
	set result "<table border=1>"
	set input [request::data]
	set pairs [split $input &]
	foreach pair $pairs {
	    if {0 == [regexp (.*)=(.*) $pair dummy varname val]} {
		# if no match, unquote and leave it at that
		# this is typical of <isindex>-type queries
		set varname anonymous
		set val $pair
	    }
	    set varname [unquote_input $varname]
	    set val     [unquote_input $val]
	    append result "<tr><td><tt>$varname</tt></td>"
	    append result "<td><tt>$val</tt></td></tr>"
	}
	append result "</table>"
	return $result
    }
}

